"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startSmsUserConsent = exports.startSmsRetriever = exports.receiveVerificationSMS = exports.requestPhoneNumber = exports.removeAllListeners = void 0;

var _reactNative = require("react-native");

const EmitterMessages = {
  SMS_RECEIVED: 'SMS_RECEIVED',
  SMS_ERROR: 'SMS_ERROR'
};
let cb = null;
const AndroidSmsVerificationApi = _reactNative.NativeModules.AndroidSmsVerificationApi;
const eventEmitter = new _reactNative.NativeEventEmitter(_reactNative.NativeModules.AndroidSmsVerificationApi);

const onMessageSuccess = message => {
  if (typeof cb === 'function') {
    cb(null, message);
  }
};

const onMessageError = error => {
  if (typeof cb === 'function') {
    cb(Error(error), null);
  }
};

const startListeners = () => {
  removeAllListeners();
  eventEmitter.addListener(EmitterMessages.SMS_RECEIVED, onMessageSuccess);
  eventEmitter.addListener(EmitterMessages.SMS_ERROR, onMessageError);
};

const removeAllListeners = () => {
  eventEmitter.removeAllListeners(EmitterMessages.SMS_RECEIVED);
  eventEmitter.removeAllListeners(EmitterMessages.SMS_ERROR);
};

exports.removeAllListeners = removeAllListeners;

const requestPhoneNumber = requestCode => {
  return AndroidSmsVerificationApi.requestPhoneNumber(requestCode || 420);
};

exports.requestPhoneNumber = requestPhoneNumber;

const receiveVerificationSMS = callback => {
  cb = callback;
  startListeners();
};

exports.receiveVerificationSMS = receiveVerificationSMS;

const startSmsRetriever = () => {
  return AndroidSmsVerificationApi.startSmsRetriever();
};

exports.startSmsRetriever = startSmsRetriever;

const startSmsUserConsent = (senderPhoneNumber, userConsentRequestCode) => {
  return AndroidSmsVerificationApi.startSmsUserConsent(senderPhoneNumber || null, userConsentRequestCode || 69);
};

exports.startSmsUserConsent = startSmsUserConsent;
//# sourceMappingURL=index.js.map